--[[
	AnimalFix
	Specialization to fix GIANTS animal scripting!
	
	@author:  	Ifko[nator]
	@date:  	23.02.2019
	@version:	1.0
]]

AnimalHusbandry.GAME_LIMIT = 20

AnimalFix = {};
AnimalFix.currentModDirectory = g_currentModDirectory;

local function animalLoadFix()
	for _, animal in pairs(g_animalManager.animals) do
		for _, subAnimal in pairs(animal.subTypes) do	
			if not fileExists(subAnimal.dummy.filename) and subAnimal.dummy.filename ~= ""	then
				subAnimal.dummy.filename = Utils.getFilename(subAnimal.dummy.filename, AnimalFix.currentModDirectory);
			end;

			if not fileExists(subAnimal.rideableFileName) and subAnimal.rideableFileName ~= "" then
				subAnimal.rideableFileName = Utils.getFilename(subAnimal.rideableFileName, AnimalFix.currentModDirectory);
			end;
			
			for l10nTAG, text in pairs(g_i18n.texts) do
				if string.find(subAnimal.subTypeName, "Missing") then
					if string.find(subAnimal.subTypeName, l10nTAG) then
				 		subAnimal.subTypeName = text;
					end;
				end;

				if string.find(subAnimal.storeInfo.shopItemName, "Missing") then
					if string.find(subAnimal.storeInfo.shopItemName, l10nTAG) then
				 		subAnimal.storeInfo.shopItemName = text;
					end;
				end;
			end;
		end;
	end;
end;

local function palletLoadFix(self, superFunc, xmlFile, configKey, rootNode, owner)
	--print("self = " .. tostring(self) .. ", superFunc = " .. tostring(superFunc) .. ", xmlFile = " .. tostring(xmlFile) .. ", configKey = " .. tostring(configKey) .. ", rootNode = " .. tostring(rootNode) .. ", owner = " .. tostring(owner));

	local result = HusbandryModulePallets:superClass().load(self, xmlFile, configKey, rootNode, owner);

    if result ~= true then
        return false;
	end;
	
    if not hasXMLProperty(xmlFile, configKey) then
        return false;
	end;

    self.palletSpawnerStartNode = I3DUtil.indexToObject(rootNode, getXMLString(xmlFile, configKey .. "#startNode"));
    self.palletSpawnerWidthNode = I3DUtil.indexToObject(rootNode, getXMLString(xmlFile, configKey .. "#widthNode"));
    self.palletSpawnerHeightNode = I3DUtil.indexToObject(rootNode, getXMLString(xmlFile, configKey .. "#heightNode"));
	
	local xw, yw, zw = getTranslation(self.palletSpawnerWidthNode);
	
    if xw == 0.0 or math.abs(yw) > 0.001 or math.abs(zw) > 0.001 then
		g_logManager:devWarning(string.format("Warning: width node of husbandry module pallets has incorrect parameters x(%.3f), y(%.3f), z(%.3f).", xw, yw, zw));
		
        return false;
	end;
	
	local xh, yh, zh = getTranslation(self.palletSpawnerHeightNode);
	
    if math.abs(xh) > 0.001 or math.abs(yh) > 0.001 or zh == 0.0 then
        g_logManager:devWarning(string.format("Warning: height node of husbandry module pallets has incorrect parameters x(%.3f), y(%.3f), z(%.3f).", xh, yh, zh));
		
		return false;
	end;
	
    self.palletSpawnerNode = self.palletSpawnerStartNode;
    self.palletSpawnerAreaSizeX = xw;
	self.palletSpawnerAreaSizeZ = zh;
	
    local fillTypeStr = getXMLString(xmlFile, configKey .. "#fillType");
	
	self.palletFillTypeIndex = g_fillTypeManager:getFillTypeIndexByName(fillTypeStr);
    self.palletConfigFilename = Utils.getFilename(Utils.getNoNil(getXMLString(xmlFile, configKey .. "#filename"), ""), AnimalFix.currentModDirectory);
    self.palletFillUnitIndex = getXMLFloat(xmlFile, configKey .. "#fillUnitIndex");
	
	self.sizeWidth, self.sizeLength, _, _ = StoreItemUtil.getSizeValues(self.palletConfigFilename, "vehicle", 0, {});
	
	self.fillLevels[self.palletFillTypeIndex] = 0.0;
	self.palletSpawnerFillDelta = 0;
    self.numObjectsInPalletSpawnerTrigger = 0;
	
	return self.palletSpawnerNode ~= nil and self.palletFillTypeIndex ~= nil and self.palletConfigFilename ~= "" and self.palletFillUnitIndex ~= nil;
end;

AnimalManager.loadAnimals = Utils.appendedFunction(AnimalManager.loadAnimals, animalLoadFix);
HusbandryModulePallets.load = Utils.overwrittenFunction(HusbandryModulePallets.load, palletLoadFix);

StableManureArea = {};
addModEventListener(StableManureArea);

function StableManureArea:loadMap(name)
		
		HusbandryModuleManure.updateManure = StableManureArea.updateManure
		
end;

function StableManureArea:deleteMap()
end;

function StableManureArea:keyEvent(unicode, sym, modifier, isDown)
end;

function StableManureArea:mouseEvent(posX, posY, isDown, isUp, button)
end;

function StableManureArea:update(dt)
end;

function StableManureArea:draw()	
end;

function StableManureArea:updateManure(manureIncrease)
	
	if self.manureArea == nil then
        return;
    end

    local manureDropped = 0;
    local fillType = self.fillTypeIndex;
    local manureArea = self.manureArea;

    if manureIncrease > g_densityMapHeightManager:getMinValidLiterValue(fillType) then
        local xs,_,zs = getWorldTranslation(manureArea.start);
        local xw,_,zw = getWorldTranslation(manureArea.width);
        local xh,_,zh = getWorldTranslation(manureArea.height);

        local ux, uz = xw-xs, zw-zs;
        local vx, vz = xh-xs, zh-zs;

        local vLength = MathUtil.vector2Length(vx,vz);

        local sx = xs + (math.random()*ux) + (math.random()*vx);
        local sz = zs + (math.random()*uz) + (math.random()*vz);

        local ex = xs + (math.random()*ux) + (math.random()*vx);
        local ez = zs + (math.random()*uz) + (math.random()*vz);

        local sy = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, sx,0,sz);
        local ey = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, ex,0,ez);
		
		local dropped, lineOffset = DensityMapHeightUtil.tipToGroundAroundLine(nil, manureIncrease, fillType, sx,sy,sz, ex,ey,ez, 0, vLength, self.lineOffsetManure, false, nil)
        manureDropped = dropped;
        self.lineOffsetManure = lineOffset;
    end
	
	return manureDropped;
end

